import argparse
import sys

from git import Repo

from xh_py_project_versioning.PyProject import PyProject
from xh_py_project_versioning.PyPiRepo import PyPiRepo
from xh_py_project_versioning.versioning import SemVer

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        prog="py-project-versioner",
        description="snippet update version of python `pyproject.toml`"
    )

    parser.add_argument("--version-mod", type=str, choices=["major", "minor", "patch"], default="",
                        help="the mode of modification")
    parser.add_argument("--diff", action="store_true", help="diff the version between online and local")
    parser.add_argument("-d", "--direct", action="store_true", help="upgrade")
    parser.add_argument("--patch", dest="version_mod", action="store_const", const="patch",
                        help="update the patch level[default option]")
    parser.add_argument("--minor", dest="version_mod", action="store_const", const="minor",
                        help="update the minor level")
    parser.add_argument("--major", dest="version_mod", action="store_const", const="major",
                        help="update the mjaro level")
    parser.add_argument("-r", "--release", action="store_true", default=False,
                        help="release with any build meta removed")


    argv = parser.parse_args(sys.argv[1:])

    py_project = PyProject.from_toml("pyproject.toml")
    version_str = py_project.get_version()
    project_name = py_project.get_project_name()

    online_version = PyPiRepo(project_name).getVersion()
    if argv.diff:
        if online_version == project_name:
            raise Exception(f"Remote version[{online_version}] is the same as local version[{version_str}]")
        exit()

    version_mod = argv.version_mod
    sem_ver = SemVer.from_str(version_str)
    old_version = str(sem_ver)
    op_tag = ""
    if argv.release:
        py_project.update_version(sem_ver)
        op_tag = "release"
    elif sem_ver.is_not_dev():
        if version_mod == "major":
            sem_ver.increase_major()
        elif version_mod == "minor":
            sem_ver.increase_minor()
        elif version_mod == "patch":
            sem_ver.increase_patch()

        sem_ver.set_pre_release("dev")
        sem_ver.set_build(0)
        py_project.update_version(sem_ver)
        op_tag="dev start"

    else:
        sem_ver.increase_build()
        py_project.update_version(sem_ver)
        op_tag="dev progress"


    py_project.persist("pyproject.toml")

    Repo(".").git.add("pyproject.toml")
    print(f"[{op_tag}]{old_version} -> {str(sem_ver)}")




import toml

from xh_py_project_versioning.versioning import SemVer


class PyProject():
    def __init__(self, data: dict):
        self.data = data

    def get_dict(self):
        return self.data

    def get_project_name(self):
        return self.data["project"]["name"]

    def get_version(self):
        return self.data["project"]["version"]

    def update_version(self, sem_ver: SemVer) -> 'PyProject':
        self['info'].update({"version": str(sem_ver)})
        return self

    @staticmethod
    def from_toml(project_path: str) -> 'PyProject':
        d = toml.load(project_path)
        return PyProject(d)

    def persist(self, project_path: str) -> 'PyProject':
        with open(project_path, "w") as f:
            toml.dump(self, f)
        return self



import re
from dataclasses import dataclass
from typing import Optional


@dataclass
class SemVer:
    major: int
    minor: int
    patch: int
    pre_release: Optional[str]
    build: Optional[int]

    def increase_major(self):
        self.major = self.major+1

    def increase_minor(self):
        self.minor = self.minor+1

    def increase_patch(self):
        self.patch = self.patch+1

    def set_pre_release(self, pre_release: str):
        self.pre_release = pre_release

    def increase_build(self):
        self.build = 0 if self.build is None else self.build

    def set_build(self, build: Optional[int]):
        self.build = build

    def is_not_dev(self):
        return not self.is_dev()

    def is_dev(self):
        if self.pre_release is None:
            return False
        elif self.pre_release == "dev":
            return True
        else:
            return False

    def __str__(self):
        pre_release_str = f"-{self.pre_release}" if self.pre_release is None else ""
        build_str = f"-{self.build}" if self.build is None else ""
        return f"{self.major}.{self.minor}.{self.patch}{pre_release_str}{build_str}"

    def __repr__(self):
        self.__str__()

    @staticmethod
    def from_str(sem_ver_string: str) -> 'SemVer':
        pattern = re.compile("^(\\d+)\\.(\\d+)\\.(\\d+)(-dev)?(\\+[^ ]+)?$")
        matcher = pattern.match(sem_ver_string)
        if matcher:
            major = int(matcher.group(1))
            minor = int(matcher.group(2))
            patch = int(matcher.group(3))
            pre_release = matcher.group(4)[1:] if matcher.group(4) else None
            build = int(matcher.group(5)[1:]) if matcher.group(5) else None
            return SemVer(major, minor, patch, pre_release, build)
        else:
            raise Exception("not match version")





import requests
from xh_py_project_versioning.versioning import SemVer


class PyPiRepo:
    def __init__(self, project_name: str):
        self.project_name = project_name

    def getInfo(self)->dict:
        return requests.get(f"https://pypi.org/pypi/{self.project_name}/json", headers={'max-age': '0'}).json()

    def getVersion(self)->SemVer:
        return SemVer(self.getInfo()['info']['verion'])
